@@ -12,7 +12,7 @@ try:
 except ImportError:
     import Queue as queue
 
-ISPY3 = sys.version_info > (3, 0)
+ISPY3 = sys.version_info >= (3, 0)
 if ISPY3:
     exec("def do_exec(co, loc): exec(co, loc)\n"
          "def reraise(cls, val, tb): raise val\n")
@@ -75,6 +75,7 @@ class Popen2IO:
             except (AttributeError, IOError):
                 pass
         self._read = getattr(infile, "buffer", infile).read
+        self._write = getattr(outfile, "buffer", outfile).write
 
     def read(self, numbytes):
         """Read exactly 'numbytes' bytes from the pipe. """
@@ -90,10 +91,7 @@ class Popen2IO:
     def write(self, data):
         """write out all data bytes. """
         assert isinstance(data, bytes)
-        try:
-            self.outfile.buffer.write(data)
-        except AttributeError:
-            self.outfile.write(data)
+        self._write(data)
         self.outfile.flush()
 
     def close_read(self):
@@ -110,12 +108,14 @@ class Message:
         self.channelid = channelid
         self.data = data
 
-    def writeto(self, serializer):
-        serializer.save((self.msgtype, self.channelid, self.data))
+    def writeto(self, io):
+        serialize(io, (self.msgtype, self.channelid, self.data))
 
-    def readfrom(cls, unserializer):
+    def readfrom(cls, io, channelfactory):
+        unserializer = Unserializer(io, channelfactory)
         msgtype, senderid, data = unserializer.load()
         return cls._types[msgtype](senderid, data)
+
     readfrom = classmethod(readfrom)
 
     def __repr__(self):
@@ -171,7 +171,7 @@ def _setupmessages():
             raise SystemExit(0)
 
     classes = [
-        STATUS, CHANNEL_EXEC, CHANNEL_DATA, CHANNEL_CLOSE, 
+        STATUS, CHANNEL_EXEC, CHANNEL_DATA, CHANNEL_CLOSE,
         CHANNEL_CLOSE_ERROR, CHANNEL_LAST_MESSAGE, GATEWAY_TERMINATE
     ]
     for i, cls in enumerate(classes):
@@ -181,7 +181,7 @@ def _setupmessages():
 
 _setupmessages()
 
-def geterrortext(excinfo, 
+def geterrortext(excinfo,
     format_exception=traceback.format_exception, sysex=sysex):
     try:
         l = format_exception(*excinfo)
@@ -330,9 +330,9 @@ class Channel(object):
         raise ValueError("mode %r not availabe" %(mode,))
 
     def close(self, error=None):
-        """ close down this channel with an optional error message. 
-            Note that closing of a channel tied to remote_exec happens 
-            automatically at the end of execution and cannot be done explicitely. 
+        """ close down this channel with an optional error message.
+            Note that closing of a channel tied to remote_exec happens
+            automatically at the end of execution and cannot be done explicitely.
         """
         if self._executing:
             raise IOError("cannot explicitly close channel within remote_exec")
@@ -364,10 +364,10 @@ class Channel(object):
         """ wait until this channel is closed (or the remote side
         otherwise signalled that no more data was being sent).
         The channel may still hold receiveable items, but not receive
-        any more after waitclose() has returned.  Exceptions from executing 
+        any more after waitclose() has returned.  Exceptions from executing
         code on the other side are reraised as local channel.RemoteErrors.
-        EOFError is raised if the reading-connection was prematurely closed, 
-        which often indicates a dying process. 
+        EOFError is raised if the reading-connection was prematurely closed,
+        which often indicates a dying process.
         self.TimeoutError is raised after the specified number of seconds
         (default is None, i.e. wait indefinitely).
         """
@@ -381,9 +381,9 @@ class Channel(object):
     def send(self, item):
         """sends the given item to the other side of the channel,
         possibly blocking if the sender queue is full.
-        The item must be a simple python type and will be 
-        copied to the other side by value.  IOError is 
-        raised if the write pipe was prematurely closed. 
+        The item must be a simple python type and will be
+        copied to the other side by value.  IOError is
+        raised if the write pipe was prematurely closed.
         """
         if self.isclosed():
             raise IOError("cannot send to %r" %(self,))
@@ -392,10 +392,10 @@ class Channel(object):
 
     def receive(self, timeout=-1):
         """receive a data item that was sent from the other side.
-        timeout: -1 [default] blocked waiting, but wake up periodically 
+        timeout: -1 [default] blocked waiting, but wake up periodically
         to let CTRL-C through.  A positive number indicates the
         number of seconds after which a channel.TimeoutError exception
-        will be raised if no item was received. 
+        will be raised if no item was received.
         Note that exceptions from the remotely executing code will be
         reraised as channel.RemoteError exceptions containing
         a textual representation of the remote traceback.
@@ -407,8 +407,8 @@ class Channel(object):
             internal_timeout = self._INTERNALWAKEUP
         else:
             internal_timeout = timeout
-           
-        while 1: 
+
+        while 1:
             try:
                 x = itemqueue.get(timeout=internal_timeout)
                 break
@@ -455,7 +455,7 @@ class ChannelFactory(object):
                 id = self.count
                 self.count += 2
             try:
-                channel = self._channels[id] 
+                channel = self._channels[id]
             except KeyError:
                 channel = self._channels[id] = Channel(self.gateway, id)
             return channel
@@ -487,17 +487,18 @@ class ChannelFactory(object):
             # channel already in "deleted" state
             if remoteerror:
                 remoteerror.warn()
+            self._no_longer_opened(id)
         else:
             # state transition to "closed" state
             if remoteerror:
                 channel._remoteerrors.append(remoteerror)
-            if not sendonly: # otherwise #--> "sendonly"
-                channel._closed = True          # --> "closed"
-            channel._receiveclosed.set()
             queue = channel._items
             if queue is not None:
                 queue.put(ENDMARKER)
-        self._no_longer_opened(id)
+            self._no_longer_opened(id)
+            if not sendonly: # otherwise #--> "sendonly"
+                channel._closed = True          # --> "closed"
+            channel._receiveclosed.set()
 
     def _local_receive(self, id, data):
         # executes in receiver thread
@@ -594,9 +595,8 @@ class BaseGateway(object):
         self.id = id
         self._channelfactory = ChannelFactory(self, _startcount)
         self._receivelock = threading.RLock()
-        self._serializer = Serializer(io)
         # globals may be NONE at process-termination
-        self._trace = trace  
+        self._trace = trace
         self._geterrortext = geterrortext
 
     def _trace(self, *msg):
@@ -613,9 +613,8 @@ class BaseGateway(object):
         eof = False
         try:
             try:
-                unserializer = Unserializer(self._io, self._channelfactory)
                 while 1:
-                    msg = Message.readfrom(unserializer)
+                    msg = Message.readfrom(self._io, self._channelfactory)
                     self._trace("received", msg)
                     _receivelock = self._receivelock
                     _receivelock.acquire()
@@ -629,7 +628,7 @@ class BaseGateway(object):
                 self._io.close_read()
             except EOFError:
                 self._trace("RECEIVERTHREAD: got EOFError")
-                self._trace("RECEIVERTHREAD: traceback was: ", 
+                self._trace("RECEIVERTHREAD: traceback was: ",
                     self._geterrortext(self.exc_info()))
                 self._error = self.exc_info()[1]
                 eof = True
@@ -642,7 +641,7 @@ class BaseGateway(object):
                     self._terminate_execution()
                 self._channelfactory._finished_receiving()
                 self._trace('RECEIVERTHREAD', 'leaving finalization')
-            except Exception:
+            except:
                 pass # XXX be silent at interp-shutdown
 
     def _terminate_execution(self):
@@ -650,7 +649,7 @@ class BaseGateway(object):
 
     def _send(self, msg):
         assert isinstance(msg, Message)
-        msg.writeto(self._serializer)
+        msg.writeto(self._io)
         self._trace('sent', msg)
 
     def _local_schedulexec(self, channel, sourcetask):
@@ -710,21 +709,35 @@ class SlaveGateway(BaseGateway):
                 self._trace("io.close_write()")
                 self._io.close_write()
                 self._trace("slavegateway.serve finished")
+            if joining:
+                self.join()
         except KeyboardInterrupt:
             # in the slave we can't really do anything sensible
             self._trace("swallowing keyboardinterrupt in main-thread")
-        if joining:
-            self.join()
 
     def executetask(self, item):
-        channel, source = item
         try:
+            channel, (source, call_name, kwargs) = item
+            if not ISPY3 and kwargs:
+                # some python2 versions do not accept unicode keyword params
+                # note: Unserializer generally turns py2-str to py3-str objects
+                newkwargs = {}
+                for name, value in kwargs.items():
+                    if isinstance(name, unicode):
+                        name = name.encode('ascii')
+                    newkwargs[name] = value
+                kwargs = newkwargs
             loc = {'channel' : channel, '__name__': '__channelexec__'}
-            self._trace("execution starts[%s]: %s" % (channel.id, repr(source)[:50]))
+            self._trace("execution starts[%s]: %s" %
+                            (channel.id, repr(source)[:50]))
             channel._executing = True
             try:
                 co = compile(source+'\n', '', 'exec')
                 do_exec(co, loc)
+                if call_name:
+                    self._trace('calling %s(**%60r)' % (call_name, kwargs))
+                    function = loc[call_name]
+                    function(channel, **kwargs)
             finally:
                 channel._executing = False
                 self._trace("execution finished")
@@ -902,10 +915,10 @@ class Unserializer(object):
         newchannel = self.channelfactory.new(id)
         self.stack.append(newchannel)
 
-# automatically build opcodes and byte-encoding 
+# automatically build opcodes and byte-encoding
 
 class opcode:
-    """ container for name -> num mappings. """ 
+    """ container for name -> num mappings. """
 
 def _buildopcodes():
     l = []
@@ -917,42 +930,45 @@ def _buildopcodes():
     for i,(opname, func) in enumerate(l):
         assert i < 26, "xxx"
         i = b(chr(64+i))
-        Unserializer.num2func[i] = func 
+        Unserializer.num2func[i] = func
         setattr(opcode, opname, i)
 
 _buildopcodes()
 
-class Serializer(object):
-    WRITE_ON_SUCCESS=True # more robust against serialize failures
+def serialize(io, obj):
+    _Serializer(io).save(obj)
+
+class _Serializer(object):
+    _dispatch = {}
 
     def __init__(self, stream):
-        self.dispatch = {}
         self._stream = stream
+        self._streamlist = []
+
+    def _write(self, data):
+        self._streamlist.append(data)
 
     def save(self, obj):
-        if self.WRITE_ON_SUCCESS:
-            self.streamlist = []
-            self._write = self.streamlist.append
-        else:
-            self._write = self.stream.write
+        # calling here is not re-entrant but multiple instances
+        # may write to the same stream because of the common platform
+        # atomic-write guaruantee (concurrent writes each happen atomicly)
         self._save(obj)
         self._write(opcode.STOP)
-        if self.WRITE_ON_SUCCESS:
-            # atomic write! (compatible to python3 and python2)
-            s = type(self.streamlist[0])().join(self.streamlist)
-            self._stream.write(s)
+        s = type(self._streamlist[0])().join(self._streamlist)
+        # atomic write
+        self._stream.write(s)
 
     def _save(self, obj):
         tp = type(obj)
         try:
-            dispatch = self.dispatch[tp]
+            dispatch = self._dispatch[tp]
         except KeyError:
             methodname = 'save_' + tp.__name__
-            meth = getattr(self, methodname, None)
+            meth = getattr(self.__class__, methodname, None)
             if meth is None:
                 raise SerializationError("can't serialize %s" % (tp,))
-            dispatch = self.dispatch[tp] = meth
-        dispatch(obj)
+            dispatch = self._dispatch[tp] = meth
+        dispatch(self, obj)
 
     def save_NoneType(self, non):
         self._write(opcode.NONE)
