@@ -61,12 +61,13 @@ class Group:
         Valid types: ``popen``, ``ssh=hostname``, ``socket=host:port``.
         Valid configuration::
 
-            id=<string>     specifies the gateway id 
+            id=<string>     specifies the gateway id
             python=<path>   specifies which python interpreter to execute
             chdir=<path>    specifies to which directory to change
             nice=<path>     specifies process priority of new process
+            env:NAME=value  specifies a remote environment variable setting.
 
-        If no spec is given, self.defaultspec is used. 
+        If no spec is given, self.defaultspec is used.
         """
         if not spec:
             spec = self.defaultspec
@@ -76,7 +77,7 @@ class Group:
         if spec.popen:
             gw = gateway.PopenGateway(python=spec.python, id=id)
         elif spec.ssh:
-            gw = gateway.SshGateway(spec.ssh, remotepython=spec.python, 
+            gw = gateway.SshGateway(spec.ssh, remotepython=spec.python,
                                     ssh_config=spec.ssh_config, id=id)
         elif spec.socket:
             assert not spec.python, (
@@ -93,19 +94,22 @@ class Group:
             raise ValueError("no gateway type found for %r" % (spec._spec,))
         gw.spec = spec
         self._register(gw)
-        if spec.chdir or spec.nice:
+        if spec.chdir or spec.nice or spec.env:
             channel = gw.remote_exec("""
                 import os
-                path, nice = channel.receive()
+                path, nice, env = channel.receive()
                 if path:
                     if not os.path.exists(path):
                         os.mkdir(path)
                     os.chdir(path)
                 if nice and hasattr(os, 'nice'):
                     os.nice(nice)
+                if env:
+                    for name, value in env.items():
+                        os.environ[name] = value
             """)
             nice = spec.nice and int(spec.nice) or 0
-            channel.send((spec.chdir, nice))
+            channel.send((spec.chdir, nice, spec.env))
             channel.waitclose()
         return gw
 
@@ -133,10 +137,10 @@ class Group:
         self.terminate(timeout=1.0)
 
     def terminate(self, timeout=None):
-        """ trigger exit of member gateways and wait for termination 
-        of member gateways and associated subprocesses.  After waiting 
-        timeout seconds try to to kill local sub processes of popen- 
-        and ssh-gateways.  Timeout defaults to None meaning 
+        """ trigger exit of member gateways and wait for termination
+        of member gateways and associated subprocesses.  After waiting
+        timeout seconds try to to kill local sub processes of popen-
+        and ssh-gateways.  Timeout defaults to None meaning
         open-ended waiting and no kill attempts.
         """
         for gw in self:
@@ -155,7 +159,7 @@ class Group:
         try:
             reply.get(timeout=timeout)
         except IOError:
-            trace("Gateways did not come down after timeout: %r" 
+            trace("Gateways did not come down after timeout: %r"
                   %(self._gateways_to_join))
             while self._gateways_to_join:
                 gw = self._gateways_to_join.pop(0)
@@ -181,10 +185,10 @@ class MultiChannel:
 
     def __iter__(self):
         return iter(self._channels)
-    
+
     def __getitem__(self, key):
         return self._channels[key]
-    
+
     def __contains__(self, chan):
         return chan in self._channels
 
@@ -242,12 +246,12 @@ def killpopen(popen):
 def killpid(pid):
     if hasattr(os, 'kill'):
         os.kill(pid, 15)
-    elif sys.platform == "win32":
+    elif sys.platform == "win32" or getattr(os, '_name', None) == 'nt':
         try:
             import ctypes
         except ImportError:
             import subprocess
-            # T: treekill, F: Force 
+            # T: treekill, F: Force
             cmd = ("taskkill /T /F /PID %d" %(pid)).split()
             ret = subprocess.call(cmd)
             if ret != 0:
@@ -259,7 +263,7 @@ def killpid(pid):
             ctypes.windll.kernel32.TerminateProcess(handle, -1)
             ctypes.windll.kernel32.CloseHandle(handle)
     else:
-        raise EnvironmmentError("no method to kill %s" %(pid,))
+        raise EnvironmentError("no method to kill %s" %(pid,))
 
 default_group = Group()
 makegateway = default_group.makegateway
