@@ -265,7 +265,7 @@ class TestChannelBasicBehaviour:
         """)
         subchan = channel.receive()
         subchan.send(1)
-        excinfo = py.test.raises(subchan.RemoteError, 
+        excinfo = py.test.raises(subchan.RemoteError,
             "subchan.waitclose(TESTTIMEOUT)")
         assert "42" in excinfo.value.formatted
         channel.send(1)
@@ -338,67 +338,5 @@ class TestChannelFile:
         channel = gw.newchannel()
         py.test.raises(ValueError, 'channel.makefile("rw")')
 
-def test_socket_gw_host_not_found(gw):
-    py.test.raises(execnet.HostNotFound,
-            'execnet.makegateway("socket=qwepoipqwe:9000")'
-    )
-
-class TestSshPopenGateway:
-    gwtype = "ssh"
-
-    def test_sshconfig_config_parsing(self, monkeypatch):
-        import subprocess
-        l = []
-        monkeypatch.setattr(subprocess, 'Popen',
-            lambda *args, **kwargs: l.append(args[0]))
-        py.test.raises(AttributeError,
-            """execnet.makegateway("ssh=xyz//ssh_config=qwe")""")
-        assert len(l) == 1
-        popen_args = l[0]
-        i = popen_args.index('-F')
-        assert popen_args[i+1] == "qwe"
-
-    def test_sshaddress(self, gw, specssh):
-        assert gw.remoteaddress == specssh.ssh
-
-    def test_host_not_found(self, gw):
-        py.test.raises(execnet.HostNotFound,
-            "execnet.makegateway('ssh=nowhere.codespeak.net')")
-
-class TestThreads:
-    def test_threads(self):
-        gw = execnet.makegateway('popen')
-        gw.remote_init_threads(3)
-        c1 = gw.remote_exec("channel.send(channel.receive())")
-        c2 = gw.remote_exec("channel.send(channel.receive())")
-        c2.send(1)
-        res = c2.receive()
-        assert res == 1
-        c1.send(42)
-        res = c1.receive()
-        assert res == 42
-
-    def test_status_with_threads(self):
-        gw = execnet.makegateway('popen')
-        gw.remote_init_threads(3)
-        c1 = gw.remote_exec("channel.send(1) ; channel.receive()")
-        c2 = gw.remote_exec("channel.send(2) ; channel.receive()")
-        c1.receive()
-        c2.receive()
-        rstatus = gw.remote_status()
-        assert rstatus.numexecuting == 2 + 1
-        assert rstatus.execqsize == 0
-        c1.send(1)
-        c2.send(1)
-        c1.waitclose()
-        c2.waitclose()
-        rstatus = gw.remote_status()
-        assert rstatus.numexecuting == 0 + 1
-        assert rstatus.execqsize == 0
-
-    def test_threads_twice(self):
-        gw = execnet.makegateway('popen')
-        gw.remote_init_threads(3)
-        py.test.raises(IOError, gw.remote_init_threads, 3)
 
 
