@@ -22,9 +22,9 @@ def test_deprecation(recwarn, monkeypatch):
 class TestBasicGateway:
     def test_correct_setup(self, gw):
         assert gw.hasreceiver()
-        assert gw in gw._group 
-        assert gw.id in gw._group 
-        assert gw.spec 
+        assert gw in gw._group
+        assert gw.id in gw._group
+        assert gw.spec
 
     def test_repr_doesnt_crash(self, gw):
         assert isinstance(repr(gw), str)
@@ -64,7 +64,7 @@ class TestBasicGateway:
         status = gw.remote_status()
         assert status.execqsize == 0
         assert status.numexecuting == 0
-        # race condition 
+        # race condition
         assert status.numchannels <= numchannels + 1
 
     def test_remote_exec_module(self, tmpdir, gw):
@@ -75,6 +75,10 @@ class TestBasicGateway:
         channel = gw.remote_exec(mod)
         name = channel.receive()
         assert name == 1
+        p.write("channel.send(2)")
+        channel = gw.remote_exec(mod)
+        name = channel.receive()
+        assert name == 2
 
     def test_correct_setup_no_py(self, gw):
         channel = gw.remote_exec("""
@@ -104,7 +108,7 @@ class TestBasicGateway:
 
     def test_remote_exec_no_explicit_close(self, gw):
         channel = gw.remote_exec('channel.close()')
-        excinfo = py.test.raises(channel.RemoteError, 
+        excinfo = py.test.raises(channel.RemoteError,
             "channel.waitclose(TESTTIMEOUT)")
         assert "explicit" in excinfo.value.formatted
 
@@ -179,12 +183,12 @@ class TestPopenGateway:
             waschangedir = old.chdir()
         c = gw.remote_exec("import os ; channel.send(os.getcwd())")
         x = c.receive()
-        assert x == str(waschangedir)
+        assert x.lower() == str(waschangedir).lower()
 
     def test_remoteerror_readable_traceback(self, gw):
-        e = py.test.raises(gateway_base.RemoteError, 
+        e = py.test.raises(gateway_base.RemoteError,
             'gw.remote_exec("x y").waitclose()')
-        assert "gateway_base" in e.value.formatted 
+        assert "gateway_base" in e.value.formatted
 
     def test_many_popen(self):
         num = 4
@@ -277,6 +281,27 @@ class TestThreads:
         res = c1.receive()
         assert res == 42
 
+    def test_threads_race_sending(self):
+        # multiple threads sending data in parallel
+        gw = execnet.makegateway("popen")
+        num = 5
+        gw.remote_init_threads(num)
+        print ("remote_init_threads(%d)" % num)
+        channels = []
+        for x in range(num):
+            ch = gw.remote_exec("""
+                for x in range(10):
+                    channel.send(''*1000)
+                channel.receive()
+            """)
+            channels.append(ch)
+        for ch in channels:
+            for x in range(10):
+                ch.receive(TESTTIMEOUT)
+            ch.send(1)
+        for ch in channels:
+            ch.waitclose(TESTTIMEOUT)
+
     def test_status_with_threads(self):
         gw = execnet.makegateway('popen')
         gw.remote_init_threads(3)
@@ -301,7 +326,7 @@ class TestThreads:
         py.test.raises(IOError, gw.remote_init_threads, 3)
 
 
-class TestTracing:        
+class TestTracing:
     def test_popen_filetracing(self, testdir, monkeypatch):
         tmpdir = testdir.tmpdir
         monkeypatch.setenv("TMP", tmpdir)
