@@ -51,13 +51,15 @@ def test_termination_on_remote_channel_receive(monkeypatch):
     gw.remote_exec("channel.receive()")
     group.terminate()
     command = ["ps", "-p", str(pid)]
-    popen = subprocess.Popen(command, stdout=subprocess.PIPE, 
+    popen = subprocess.Popen(command, stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT)
     out, err = popen.communicate()
     out = py.builtin._totext(out, 'utf8')
     assert str(pid) not in out, out
 
 def test_close_initiating_remote_no_error(testdir, anypython):
+    if '2.4' in str(anypython):
+        py.test.xfail("race/wait/interrupt_main/thread-loop issue with python2.4")
     p = testdir.makepyfile("""
         import sys
         sys.path.insert(0, %r)
@@ -69,7 +71,7 @@ def test_close_initiating_remote_no_error(testdir, anypython):
         ch3 = gw.remote_exec("channel.receive()")
         execnet.default_group.terminate()
     """ % str(execnetdir))
-    popen = subprocess.Popen([str(anypython), str(p)], 
+    popen = subprocess.Popen([str(anypython), str(p)],
             stdout=subprocess.PIPE, stderr=subprocess.PIPE,)
     stdout, stderr = popen.communicate()
     print (stdout)
@@ -94,6 +96,6 @@ def test_terminate_implicit_does_trykill(testdir, anypython, capfd):
     # sync with start-up
     line = popen.stdout.readline()
     reply = WorkerPool(1).dispatch(popen.communicate)
-    reply.get(timeout=10)
+    reply.get(timeout=50)
     out, err = capfd.readouterr()
     assert not err or "Killed" in err
